@@ -97,7 +97,7 @@ impl BinaryConfusionMatrix {
     ///
     /// An invalid input error will be returned if all provided counts are zero
     ///
-    pub fn with_counts(tp_count: usize,
+    pub fn from_counts(tp_count: usize,
                        fp_count: usize,
                        tn_count: usize,
                        fn_count: usize) -> Result<BinaryConfusionMatrix, EvalError> {
@@ -536,18 +536,18 @@ impl MultiConfusionMatrix {
     ///     vec![1, 5, 3],
     ///     vec![2, 1, 9]
     /// ];
-    /// let matrix = MultiConfusionMatrix::with_counts(counts)?;
+    /// let matrix = MultiConfusionMatrix::from_counts(counts)?;
     /// # Ok(())}
     /// ```
     ///
-    pub fn with_counts(counts: Vec<Vec<usize>>) -> Result<MultiConfusionMatrix, EvalError> {
+    pub fn from_counts(counts: Vec<Vec<usize>>) -> Result<MultiConfusionMatrix, EvalError> {
         let dim = counts.len();
         let mut sum = 0;
         for row in &counts {
             sum += row.iter().sum::<usize>();
             if row.len() != dim {
                 let msg = format!("Inconsistent column length ({})", row.len());
-                return Err(EvalError::InvalidInput(msg));
+                return Err(EvalError::invalid_input(msg.as_str()));
             }
         }
         if sum == 0 {
@@ -639,31 +639,66 @@ impl MultiConfusionMatrix {
         }
     }
 
+    ///
+    /// Computes per-class accuracy, resulting in a vector of values for each class
+    ///
+    pub fn per_class_accuracy(&self) -> Vec<Result<f64, EvalError>> {
+        self.per_class_binary_metric("accuracy")
+    }
+
     ///
     /// Computes per-class precision, resulting in a vector of values for each class
     ///
     pub fn per_class_precision(&self) -> Vec<Result<f64, EvalError>> {
-        self.per_class_pr_metric(PrMetric::Precision)
+        self.per_class_binary_metric("precision")
     }
 
     ///
     /// Computes per-class recall, resulting in a vector of values for each class
     ///
     pub fn per_class_recall(&self) -> Vec<Result<f64, EvalError>> {
-        self.per_class_pr_metric(PrMetric::Recall)
+        self.per_class_binary_metric("recall")
     }
 
     ///
     /// Computes per-class F1, resulting in a vector of values for each class
     ///
     pub fn per_class_f1(&self) -> Vec<Result<f64, EvalError>> {
-        let pcp = self.per_class_precision();
-        let pcr = self.per_class_recall();
-        pcp.iter().zip(pcr.iter()).map(|pair| {
-            match pair {
-                (Ok(p), Ok(r)) if *p == 0.0 && *r == 0.0 => Ok(0.0),
-                (Ok(p), Ok(r)) => Ok(2.0 * (p * r) / (p + r)),
-                (Err(e), _) | (_, Err(e)) => Err(e.clone())
+        self.per_class_binary_metric("f1")
+    }
+
+    ///
+    /// Computes per-class MCC, resulting in a vector of values for each class
+    ///
+    pub fn per_class_mcc(&self) -> Vec<Result<f64, EvalError>> {
+        self.per_class_binary_metric("mcc")
+    }
+
+    fn per_class_binary_metric(&self, metric: &str) -> Vec<Result<f64, EvalError>> {
+        (0..self.dim).map(|k| {
+            let (mut tpc, mut fpc, mut tnc, mut fnc) = (0, 0, 0, 0);
+            for i in 0..self.dim {
+                for j in 0..self.dim {
+                    let count = self.counts[i][j];
+                    if i == k && j == k {
+                        tpc = count;
+                    } else if i == k {
+                        fpc += count;
+                    } else if j == k {
+                        fnc += count;
+                    } else {
+                        tnc += count;
+                    }
+                }
+            }
+            let matrix = BinaryConfusionMatrix::from_counts(tpc, fpc, tnc, fnc)?;
+            match metric {
+                "accuracy" => matrix.accuracy(),
+                "precision" => matrix.precision(),
+                "recall" => matrix.recall(),
+                "f1" => matrix.f1(),
+                "mcc" => matrix.mcc(),
+                other => Err(EvalError::invalid_metric(other))
             }
         }).collect()
     }
@@ -676,22 +711,6 @@ impl MultiConfusionMatrix {
         }
     }
 
-    fn per_class_pr_metric(&self, metric: PrMetric) -> Vec<Result<f64, EvalError>> {
-        (0..self.dim).map(|i| {
-            let a = self.counts[i][i];
-            let b = (0..self.dim).fold(0, |sum, j| {
-                sum + match metric {
-                    PrMetric::Precision => self.counts[i][j],
-                    PrMetric::Recall => self.counts[j][i]
-                }
-            }) - a;
-            match a + b {
-                0 => Err(EvalError::UndefinedMetric(format!("Per-Class {}", metric.name()))),
-                den => Ok(a as f64 / den as f64)
-            }
-        }).collect()
-    }
-
     fn macro_metric(&self, pcm: &Vec<Result<f64, EvalError>>) -> Result<f64, EvalError> {
         pcm.iter().try_fold(0.0, |sum, metric| {
             match metric {
@@ -812,20 +831,6 @@ pub enum Averaging {
     Weighted
 }
 
-enum PrMetric {
-    Precision,
-    Recall
-}
-
-impl PrMetric {
-    fn name(&self) -> &'static str {
-        match self {
-            PrMetric::Precision => "Precision",
-            PrMetric::Recall => "Recall"
-        }
-    }
-}
-
 enum RocTrend {
     Horizontal,
     Vertical,
@@ -926,13 +931,13 @@ mod tests {
 
     #[test]
     fn test_binary_confusion_matrix_with_counts() {
-        let matrix = BinaryConfusionMatrix::with_counts(2, 4, 5, 3).unwrap();
+        let matrix = BinaryConfusionMatrix::from_counts(2, 4, 5, 3).unwrap();
         assert_eq!(matrix.tp_count, 2);
         assert_eq!(matrix.fp_count, 4);
         assert_eq!(matrix.tn_count, 5);
         assert_eq!(matrix.fn_count, 3);
         assert_eq!(matrix.sum, 14);
-        assert!(BinaryConfusionMatrix::with_counts(0, 0, 0, 0).is_err())
+        assert!(BinaryConfusionMatrix::from_counts(0, 0, 0, 0).is_err())
     }
 
     #[test]
@@ -1278,7 +1283,7 @@ mod tests {
     #[test]
     fn test_multi_confusion_matrix_counts() {
         let counts = vec![vec![6, 3, 1], vec![4, 2, 7], vec![5, 2, 8]];
-        let matrix = MultiConfusionMatrix::with_counts(counts).unwrap();
+        let matrix = MultiConfusionMatrix::from_counts(counts).unwrap();
         assert_eq!(matrix.dim, 3);
         assert_eq!(matrix.sum, 38);
         assert_eq!(matrix.counts, vec![vec![6, 3, 1], vec![4, 2, 7], vec![5, 2, 8]]);
@@ -1287,7 +1292,7 @@ mod tests {
     #[test]
     fn test_multi_confusion_matrix_bad_counts() {
         let counts = vec![vec![6, 3, 1], vec![4, 2], vec![5, 2, 8]];
-        assert!(MultiConfusionMatrix::with_counts(counts).is_err())
+        assert!(MultiConfusionMatrix::from_counts(counts).is_err())
     }
 
     #[test]
@@ -1371,6 +1376,62 @@ mod tests {
         assert_approx_eq!(matrix.rk().unwrap(), 0.375)
     }
 
+    #[test]
+    fn test_per_class_accuracy() {
+        let (scores, labels) = multi_class_data();
+        let matrix = MultiConfusionMatrix::compute(&scores, &labels).unwrap();
+        let pca = matrix.per_class_accuracy();
+        assert_eq!(pca.len(), 3);
+        assert_approx_eq!(pca[0].as_ref().unwrap(), 0.5714285714285714);
+        assert_approx_eq!(pca[1].as_ref().unwrap(), 0.7142857142857143);
+        assert_approx_eq!(pca[2].as_ref().unwrap(), 0.8571428571428571);
+    }
+
+    #[test]
+    fn test_per_class_precision() {
+        let (scores, labels) = multi_class_data();
+        let matrix = MultiConfusionMatrix::compute(&scores, &labels).unwrap();
+        let pcp = matrix.per_class_precision();
+        assert_eq!(pcp.len(), 3);
+        assert_approx_eq!(pcp[0].as_ref().unwrap(), 0.3333333333333333);
+        assert_approx_eq!(pcp[1].as_ref().unwrap(), 0.5);
+        assert_approx_eq!(pcp[2].as_ref().unwrap(), 1.0);
+        println!("{}", matrix);
+    }
+
+    #[test]
+    fn test_per_class_recall() {
+        let (scores, labels) = multi_class_data();
+        let matrix = MultiConfusionMatrix::compute(&scores, &labels).unwrap();
+        let pcr = matrix.per_class_recall();
+        assert_eq!(pcr.len(), 3);
+        assert_approx_eq!(pcr[0].as_ref().unwrap(), 0.5);
+        assert_approx_eq!(pcr[1].as_ref().unwrap(), 0.5);
+        assert_approx_eq!(pcr[2].as_ref().unwrap(), 0.6666666666666666);
+    }
+
+    #[test]
+    fn test_per_class_f1() {
+        let (scores, labels) = multi_class_data();
+        let matrix = MultiConfusionMatrix::compute(&scores, &labels).unwrap();
+        let pcf = matrix.per_class_f1();
+        assert_eq!(pcf.len(), 3);
+        assert_approx_eq!(pcf[0].as_ref().unwrap(), 0.4);
+        assert_approx_eq!(pcf[1].as_ref().unwrap(), 0.5);
+        assert_approx_eq!(pcf[2].as_ref().unwrap(), 0.8);
+    }
+
+    #[test]
+    fn test_per_class_mcc() {
+        let (scores, labels) = multi_class_data();
+        let matrix = MultiConfusionMatrix::compute(&scores, &labels).unwrap();
+        let pcm = matrix.per_class_mcc();
+        assert_eq!(pcm.len(), 3);
+        assert_approx_eq!(pcm[0].as_ref().unwrap(), 0.09128709291752773);
+        assert_approx_eq!(pcm[1].as_ref().unwrap(), 0.3);
+        assert_approx_eq!(pcm[2].as_ref().unwrap(), 0.7302967433402215);
+    }
+
     #[test]
     fn test_m_auc() {
         let (scores, labels) = multi_class_data();
